<?php
namespace NOUT\Entity\UsernameToken;

use NOUT\Soap\StructType\EncryptionType;
use NOUT\Soap\StructType\UsernameTokenType;

/**
 * Classe UserNameToken
 * permetant de générer les info de connection pour la norme OASIS (username, password, nonce, et created)

 */
abstract class UsernameToken extends UsernameTokenType implements UsernameTokenInterface
{
    /**
     * Crypte les différents éléments
     */
    abstract protected function _Compute() : void;

    /**
     * @param string $password
     */
    abstract protected function _setClearPassword(string $password) : void;

    /**
     * @param string $password
     * @return $this
     */
    final public function setClearPassword(string $password): UsernameToken
    {
        $this->_setClearPassword($password);
        return $this;
    }

    /**
     * Init Created and Nonce puis crypte
     * @throws \Exception
     */
    final public function Compute() : void
    {
        $this->Created = date('r');
        $this->Nonce   = bin2hex(random_bytes(20));

        $this->_Compute();
    }

    final public function transformForSOAP()
    {
        if ($this->Encryption instanceof EncryptionType)
        {
            $encryption = $this->Encryption;
            $this->Encryption = [
                '!' => $encryption->get_(),
                'md5' => $encryption->getMd5(),
                'ks' => $encryption->getKs(),
                'iv' => $encryption->getIv()
            ];
        }
    }

    /**
     * @return string
     * @throws \Exception
     */
    public function sToRest() : string
    {
        $this->Compute(); //on fait le compute

        $sBottom = 'Username='.urlencode(mb_convert_encoding($this->Username, 'Windows-1252', 'UTF-8'));
        $sBottom .= '&Password='.urlencode($this->Password); //pas la peine de décoder à cause des caractère utilisé
        $sBottom .= '&nonce='.urlencode(mb_convert_encoding($this->Nonce, 'Windows-1252', 'UTF-8'));
        $sBottom .= '&created='.urlencode(mb_convert_encoding($this->Created, 'Windows-1252', 'UTF-8'));

        if (!empty($this->Encryption))
        {
            $sBottom .= '&encryption=' . urlencode($this->Encryption->get_());
            $sBottom .= '&md5=' . urlencode($this->Encryption->getMd5());
            if (!empty($this->Encryption->getIv())){
                $sBottom .= '&iv=' . urlencode($this->Encryption->getIv());
            }
            if (!empty($this->Encryption->getKs())){
                $sBottom .= '&ks=' . urlencode($this->Encryption->getKs());
            }
        }
        return $sBottom;
    }

    public function __clone()
    {
        if (is_object($this->Encryption)){
            $this->Encryption = clone $this->Encryption;
        }
    }
}

