<?php

namespace NOUT\Soap\ServiceType;

use NOUT\Entity\SIMAXServiceHeader;
use NOUT\Entity\UsernameToken\UsernameTokenInterface;
use NOUT\Soap\PackageBase\AbstractSoapClientBase;
use NOUT\Soap\PackageBase\SoapClientInterface;
use NOUT\Soap\StructType\Action;

class SIMAXService extends Service
{

    /**
     * Contains input headers
     * @var array
     */
    protected $inputHeaders=[];

    const WSDL_PROXY_URI = 'wsdl_proxy_uri';

    public function __construct(array $wsdlOptions = [])
    {
        $proxyParams = [];
//        if (array_key_exists(self::WSDL_PROXY_URI, $wsdlOptions)){
//            $proxyParams = [
//                'proxy' => $wsdlOptions[self::WSDL_PROXY_URI],
//                'request_fulluri' => true,
//            ];
//            unset($wsdlOptions[self::WSDL_PROXY_URI]);
//        }
        $wsdlOptions[SoapClientInterface::WSDL_EXCEPTIONS]     = 1;

        $aStreamOpt = [
                          'http' => [
                              'header' => SIMAXServiceHeader::HEADER_SERVICE_CLIENT . ": " . SIMAXServiceHeader::CLIENTNAME . "\n" . SIMAXServiceHeader::HEADER_SERVICE_CLIENT_VERSION . ": " . SIMAXServiceHeader::CLIENTVERSION . "\n" . SIMAXServiceHeader::HEADER_SERVICE_CLIENT_IP . ": " . $_SERVER["REMOTE_ADDR"],
                          ]+$proxyParams,
//                          'ssl' => [
//                              'verify_peer' => false,
//                              'verify_peer_name' => false,
//                          ],
                      ];
        $streamctxt = stream_context_create($aStreamOpt);

        $wsdlOptions[SoapClientInterface::WSDL_STREAM_CONTEXT] = $streamctxt;
        parent::__construct($wsdlOptions);
    }


    /**
     * Returns the SoapClient class name to use to create the instance of the SoapClient.
     * Be sure that this class inherits from the native PHP SoapClient class and this class has been loaded or can be loaded.
     * The goal is to allow the override of the SoapClient without having to modify this generated class.
     * Then the overriding SoapClient class can override for example the SoapClient::__doRequest() method if it is needed.
     * @param string|null $soapClientClassName
     * @return string
     */
    public function getSoapClientClassName(?string $soapClientClassName = null): string
    {
        return parent::getSoapClientClassName(SIMAXSoap::class);
    }

    /**
     * Static method getting current SoapClient
     * @return \SoapClient|null
     */
    public function getSoapClient() : ?\SoapClient
    {
        $oSoapClient = parent::getSoapClient();
        if (!is_null($oSoapClient)){
            foreach ($this->inputHeaders as $index => $soapHeader) {
                if ($soapHeader->name === 'UsernameToken') {
                    $oUsernameToken = $this->inputHeaders[$index]->data;
                    if ($oUsernameToken instanceof UsernameTokenInterface){
                        $oUsernameToken->Compute();
                        break;
                    }
                }
            }
        }

        return $oSoapClient;
    }


    /**
     * Sets a SoapHeader to send
     * For more information, please read the online documentation on {@link http://www.php.net/manual/en/class.soapheader.php}
     * @param string $namespace SoapHeader namespace
     * @param string $name SoapHeader name
     * @param mixed $data SoapHeader data
     * @param bool $mustUnderstand
     * @param string|null $actor
     * @return SIMAXService
     */
    public function setSoapHeader(string $namespace, string $name, $data, bool $mustUnderstand = false, ?string $actor = null): self
    {
        if ($this->getSoapClient()) {
            foreach ($this->inputHeaders as $index => $soapHeader) {
                if ($soapHeader->name === $name) {
                    unset($this->inputHeaders[$index]);
                    break;
                }
            }
            $this->getSoapClient()->__setSoapheaders(null);
            if (!empty($actor)) {
                array_push($this->inputHeaders, new \SoapHeader($namespace, $name, $data, $mustUnderstand, $actor));
            } else {
                array_push($this->inputHeaders, new \SoapHeader($namespace, $name, $data, $mustUnderstand));
            }
            $this->getSoapClient()->__setSoapheaders($this->inputHeaders);
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getSoapHeaderReturnType()
    {
        return $this->outputHeaders['ReturnType'] ?? 'ERROR';
    }

    /**
     * @return string
     */
    public function getSoapHeaderActionContext()
    {
        return $this->outputHeaders['ActionContext'];
    }

    /**
     * @return string
     */
    public function getSoapHeaderXSDSchema()
    {
        return $this->outputHeaders['XSDSchema'];
    }

    /**
     * @return \NOUT\Soap\StructType\Action
     */
    public function getSoapHeaderAction() : Action
    {
        return $this->outputHeaders['Action'];
    }

    /**
     * @return \NOUT\Soap\StructType\Filter
     */
    public function getSoapHeaderFilter()
    {
        return $this->outputHeaders['Filter'];
    }

    /**
     * @return \NOUT\Soap\StructType\PlanningFilter
     */
    public function getSoapHeaderPlanningFilter()
    {
        return $this->outputHeaders['PlanningFilter'];
    }

    /**
     * @return \NOUT\Soap\StructType\ConnectedUser
     */
    public function getSoapHeaderConnectedUser()
    {
        return $this->outputHeaders['ConnectedUser'];
    }

    /**
     * @return \NOUT\Soap\StructType\ConnectedExtranet
     */
    public function getSoapHeaderConnectedExtranet()
    {
        return $this->outputHeaders['ConnectedExtranet'];
    }


    /**
     * @return \NOUT\Soap\StructType\Form
     */
    public function getSoapHeaderForm()
    {
        return $this->outputHeaders['Form'];
    }

    /**
     * @return \NOUT\Soap\StructType\Element
     */
    public function getSoapHeaderElement()
    {
        return $this->outputHeaders['Element'];
    }

    /**
     * @return \NOUT\Soap\StructType\Count
     */
    public function getSoapHeaderCount()
    {
        return $this->outputHeaders['Count'];
    }

    /**
     * @return \NOUT\Soap\StructType\ValidateError
     */
    public function getSoapHeaderValidateError()
    {
        return $this->outputHeaders['ValidateError'];
    }

    /**
     * @return integer
     */
    public function getSoapHeaderNextCall()
    {
        return $this->outputHeaders['NextCall'];
    }

    /**
     * @return integer
     */
    public function getSoapHeaderPJSizeCheck()
    {
        return $this->outputHeaders['PJSizeCheck'];
    }

    /**
     * @return \NOUT\Soap\StructType\RecipientCheck
     */
    public function getSoapHeaderRecipientCheck()
    {
        return $this->outputHeaders['RecipientCheck'];
    }

    /**
     * @return string
     */
    public function getSoapHeaderCustomerInfos()
    {
        return $this->outputHeaders['CustomerInfos'];
    }

    /**
     * @return string
     */
    public function getSoapHeaderSessionLanguageCode()
    {
        return $this->outputHeaders['SessionLanguageCode'];
    }

    /**
     * Sets the ActionContext SoapHeader param
     * @uses AbstractSoapClientBase::setSoapHeader()
     * @param string|null $actionContext
     * @param string $namespace
     * @param bool $mustUnderstand
     * @param string|null $actor
     * @return \NOUT\Soap\ServiceType\Service
     */
    public function setSoapHeaderActionContext(?string $actionContext, string $namespace = 'http://www.nout.fr/wsdl/SimaxService.wsdl/', bool $mustUnderstand = false, ?string $actor = null): self
    {
        // validation for constraint: string
        if (!is_null($actionContext) && !is_string($actionContext)) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide a string, %s given', var_export($actionContext, true), gettype($actionContext)), __LINE__);
        }
        return $this->setSoapHeader($namespace, 'ActionContext', $actionContext, $mustUnderstand, $actor);
    }

    /**
     * Sets the AutoValidate SoapHeader param
     * @uses AbstractSoapClientBase::setSoapHeader()
     * @param int|null $autoValidate
     * @param string $namespace
     * @param bool $mustUnderstand
     * @param string|null $actor
     * @return \NOUT\Soap\ServiceType\Service
     */
    public function setSoapHeaderAutoValidate(?int $autoValidate, string $namespace = 'http://www.nout.fr/wsdl/SimaxService.wsdl/', bool $mustUnderstand = false, ?string $actor = null): self
    {
        // validation for constraint: int
        if (!is_null($autoValidate) && !(is_int($autoValidate) || ctype_digit($autoValidate))) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide an integer value, %s given', var_export($autoValidate, true), gettype($autoValidate)), __LINE__);
        }
        return $this->setSoapHeader($namespace, 'AutoValidate', $autoValidate, $mustUnderstand, $actor);
    }
}
