<?php

namespace NOUT\Soap\ServiceType;

use NOUT\Soap\PackageBase\AbstractStructBase;
use NOUT\Soap\PackageBase\SoapClientInterface;
use NOUT\Soap\PackageBase\StructInterface;

class SIMAXSoap extends \SoapClient
{
    protected ?array $classmap = null;

    public function __construct(?string $wsdl, array $options = [])
    {
        $options[str_replace(SoapClientInterface::OPTION_PREFIX, '', SoapClientInterface::WSDL_TRACE)]=1; //on force les traces
        parent::__construct($wsdl, $options);
        $this->classmap = $options[str_replace(SoapClientInterface::OPTION_PREFIX, '', SoapClientInterface::WSDL_CLASSMAP)] ?? null;
    }

    /**
     * @param       $name
     * @param array $args
     * @param       $options
     * @param       $inputHeaders
     * @param       $outputHeaders
     *
     * @return mixed
     * @throws \SoapFault
     */
    public function __soapCall(
        $name,
        array $args,
        $options = null,
        $inputHeaders = null,
        &$outputHeaders = null
    )
    {
        $ret = parent::__soapCall(
            $name
            ,$args
            ,$options
            ,$inputHeaders
            ,$outputHeaders
        );

        if (empty($this->classmap)){
            return $ret;
        }

        //ici il faut gérer les outputs qui ont des attributs parce que SoapClient n'est pas fichu de le faire
        $aSoapTypes = $this->__getTypes();
        $ndHeader = $this->_ndGetHeaderNode();
        foreach($outputHeaders as $headername=>$header)
        {
            $newHeader = $this->_traiteOutputHeader($headername, $header, $aSoapTypes, $ndHeader);
            if (!is_null($newHeader)){
                $outputHeaders[$headername]=$newHeader;
            }
        }

        return $ret;
    }

    protected function _ndGetHeaderNode() : \SimpleXMLElement
    {
        $lastResponse = $this->__getLastResponse();
        $clEnvelope = simplexml_load_string($lastResponse);


        //calcul du nom du namespace de l'enveloppe
        $tabNamespaceDuXML    = $clEnvelope->getNamespaces();
        // tableau associatif nom => url

        // Il n'y a que ces 4 là qui sont possibles pour l'enveloppe SOAP (standard w3c)
        // les deux mêmes avec / final ou nom pour compatibilité avec vieille version de NOUTOnline
        $aNamespacePossible = array(
            'http://www.w3.org/2003/05/soap-envelope/',
            'http://schemas.xmlsoap.org/soap/envelope/',
            'http://www.w3.org/2003/05/soap-envelope',
            'http://schemas.xmlsoap.org/soap/envelope',
        );



        //recherche les namespaces possible dans les namespances du XML
        foreach($aNamespacePossible as $sUrlNamespacePossible)
        {
            $sNamespaceSOAP = array_search($sUrlNamespacePossible, $tabNamespaceDuXML); // false si ne trouve pas la valeur dans le tableau
            //renvoi la clé si trouvé, donc le nom du namespace

            if ($sNamespaceSOAP !== false) // Si le namespace de l'enveloppe est un de ceux autorisés
            {
                break;
            }
        }

        return $clEnvelope->children($sNamespaceSOAP, true)->Header;
    }

    protected function _oGetStructDefinition(string $name, array $aSoapTypes) : ?array
    {
        $aHeaderDefinition = array_filter($aSoapTypes, function($typedef) use($name){
            return strpos($typedef, "struct $name {") !== false;
        });

        if (empty($aHeaderDefinition)){
            return null;
        }

        $aStructTemp = explode("\n", array_shift($aHeaderDefinition));
        //on enlève la première ligne et la dernière pour avoir que les membres
        array_pop($aStructTemp);
        array_shift($aStructTemp);

        $aStruct=[];
        foreach($aStructTemp as $line)
        {
            preg_match_all('/(?<type>\w+) (?<name>\w+);/', $line, $matches, PREG_SET_ORDER, 0);

            $type = $matches[0]['type'];
            $name = $matches[0]['name'];

            $classtype = $this->classmap[$type] ?? false;
            $oSubdef = null;
            if ($classtype){
                $type = $classtype;
                $oSubdef = $this->_oGetStructDefinition($matches[0]['type'], $aSoapTypes);
            }
            $aStruct[$name] = ['type' => $type, 'subdef' => $oSubdef];
        }

        return $aStruct;
    }

    protected function _traiteOutputHeader(string $headername, $header, array $aSoapTypes, \SimpleXMLElement $ndRootHeader) : ?StructInterface
    {
        //on recupère la classe
        $classtype = $this->classmap[$headername] ?? false;
        $oStruct = $this->_oGetStructDefinition($headername, $aSoapTypes);
        if (is_null($oStruct) || !$classtype){
            return null;
        }

        if (($header instanceof $classtype) && $header->isOfType($oStruct))
        {
            return null; //tout va bien
        }

        //il faut trouver le header
        $ndHeader = $ndRootHeader->children()->$headername;


        return $this->_oCreateHeaderFromXML($oStruct, $classtype, $ndHeader);
    }

    protected function _oCreateHeaderFromXML(array $oStruct, string $classtype, \SimpleXMLElement $ndHeader) : ?StructInterface
    {
        if (!$ndHeader->count()){
            return null;
        }

        $stHeader = new $classtype();

        if (array_key_exists('_', $oStruct)){
            //structure avec des attributs
            $value = $ndHeader->__toString();
            $stHeader->set_($value);

            unset($oStruct['_']);
            $attributes = $ndHeader->attributes();
            foreach ($oStruct as $name => $info)
            {
                ['type' => $type] = $info;
                $ndAttribute = $attributes[$name];
                if (!is_null($ndAttribute)){
                    $value = $attributes[$name]->__toString();
                    if ($type != 'string'){
                        $value = "{$type}val"($value);
                    }
                    $stHeader->{"set{$name}"}($value);
                }
            }
        }
        else {
            foreach ($oStruct as $name => $info)
            {
                ['type' => $type, 'subdef' => $subdef] = $info;
                $ndFils = $ndHeader->$name;
                if (!is_null($ndFils))
                {
                    if ($type[0]=='\\'){
                        $value = $this->_oCreateHeaderFromXML($subdef, $type, $ndFils);
                    }
                    else
                    {
                        $value = $ndFils->__toString();
                        if ($type != 'string'){
                            $value = "{$type}val"($value);
                        }
                    }

                    if (method_exists($stHeader, "addTo{$name}")){
                        if (!empty($value)){
                            $stHeader->{"addTo{$name}"}($value);
                        }
                    }
                    else {
                        $stHeader->{"set{$name}"}($value);
                    }


                }

            }
        }

        return $stHeader;
    }

}
