<?php

declare(strict_types=1);

namespace NOUT\Soap\StructType;

use InvalidArgumentException;
use NOUT\Soap\PackageBase\AbstractStructBase;

/**
 * This class stands for CallingInfoType StructType
 * @subpackage Structs
 */
#[\AllowDynamicProperties]
class CallingInfoType extends AbstractStructBase
{
    /**
     * The Column
     * @var string|null
     */
    protected ?string $Column = null;
    /**
     * The Context
     * Meta information extracted from the WSDL
     * - minOccurs: 0
     * @var string|null
     */
    protected ?string $Context = null;
    /**
     * The Record
     * Meta information extracted from the WSDL
     * - minOccurs: 0
     * @var string|null
     */
    protected ?string $Record = null;
    /**
     * The Value
     * Meta information extracted from the WSDL
     * - minOccurs: 0
     * @var string|null
     */
    protected ?string $Value = null;
    /**
     * The SelectedItems
     * Meta information extracted from the WSDL
     * - maxOccurs: unbounded
     * - minOccurs: 0
     * @var \NOUT\Soap\StructType\SelectedItemsType[]
     */
    protected ?array $SelectedItems = null;
    /**
     * Constructor method for CallingInfoType
     * @uses CallingInfoType::setColumn()
     * @uses CallingInfoType::setContext()
     * @uses CallingInfoType::setRecord()
     * @uses CallingInfoType::setValue()
     * @uses CallingInfoType::setSelectedItems()
     * @param string $column
     * @param string $context
     * @param string $record
     * @param string $value
     * @param \NOUT\Soap\StructType\SelectedItemsType[] $selectedItems
     */
    public function __construct(?string $column = null, ?string $context = null, ?string $record = null, ?string $value = null, ?array $selectedItems = null)
    {
        $this
            ->setColumn($column)
            ->setContext($context)
            ->setRecord($record)
            ->setValue($value)
            ->setSelectedItems($selectedItems);
    }
    /**
     * Get Column value
     * @return string|null
     */
    public function getColumn(): ?string
    {
        return $this->Column;
    }
    /**
     * Set Column value
     * @param string $column
     * @return \NOUT\Soap\StructType\CallingInfoType
     */
    public function setColumn(?string $column = null): self
    {
        // validation for constraint: string
        if (!is_null($column) && !is_string($column)) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide a string, %s given', var_export($column, true), gettype($column)), __LINE__);
        }
        $this->Column = $column;
        
        return $this;
    }
    /**
     * Get Context value
     * @return string|null
     */
    public function getContext(): ?string
    {
        return $this->Context;
    }
    /**
     * Set Context value
     * @param string $context
     * @return \NOUT\Soap\StructType\CallingInfoType
     */
    public function setContext(?string $context = null): self
    {
        // validation for constraint: string
        if (!is_null($context) && !is_string($context)) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide a string, %s given', var_export($context, true), gettype($context)), __LINE__);
        }
        $this->Context = $context;
        
        return $this;
    }
    /**
     * Get Record value
     * @return string|null
     */
    public function getRecord(): ?string
    {
        return $this->Record;
    }
    /**
     * Set Record value
     * @param string $record
     * @return \NOUT\Soap\StructType\CallingInfoType
     */
    public function setRecord(?string $record = null): self
    {
        // validation for constraint: string
        if (!is_null($record) && !is_string($record)) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide a string, %s given', var_export($record, true), gettype($record)), __LINE__);
        }
        $this->Record = $record;
        
        return $this;
    }
    /**
     * Get Value value
     * @return string|null
     */
    public function getValue(): ?string
    {
        return $this->Value;
    }
    /**
     * Set Value value
     * @param string $value
     * @return \NOUT\Soap\StructType\CallingInfoType
     */
    public function setValue(?string $value = null): self
    {
        // validation for constraint: string
        if (!is_null($value) && !is_string($value)) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide a string, %s given', var_export($value, true), gettype($value)), __LINE__);
        }
        $this->Value = $value;
        
        return $this;
    }
    /**
     * Get SelectedItems value
     * @return \NOUT\Soap\StructType\SelectedItemsType[]
     */
    public function getSelectedItems(): ?array
    {
        return $this->SelectedItems;
    }
    /**
     * This method is responsible for validating the value(s) passed to the setSelectedItems method
     * This method is willingly generated in order to preserve the one-line inline validation within the setSelectedItems method
     * This has to validate that each item contained by the array match the itemType constraint
     * @param array $values
     * @return string A non-empty message if the values does not match the validation rules
     */
    public static function validateSelectedItemsForArrayConstraintFromSetSelectedItems(?array $values = []): string
    {
        if (!is_array($values)) {
            return '';
        }
        $message = '';
        $invalidValues = [];
        foreach ($values as $callingInfoTypeSelectedItemsItem) {
            // validation for constraint: itemType
            if (!$callingInfoTypeSelectedItemsItem instanceof \NOUT\Soap\StructType\SelectedItemsType) {
                $invalidValues[] = is_object($callingInfoTypeSelectedItemsItem) ? get_class($callingInfoTypeSelectedItemsItem) : sprintf('%s(%s)', gettype($callingInfoTypeSelectedItemsItem), var_export($callingInfoTypeSelectedItemsItem, true));
            }
        }
        if (!empty($invalidValues)) {
            $message = sprintf('The SelectedItems property can only contain items of type \NOUT\Soap\StructType\SelectedItemsType, %s given', is_object($invalidValues) ? get_class($invalidValues) : (is_array($invalidValues) ? implode(', ', $invalidValues) : gettype($invalidValues)));
        }
        unset($invalidValues);
        
        return $message;
    }
    /**
     * Set SelectedItems value
     * @throws InvalidArgumentException
     * @param \NOUT\Soap\StructType\SelectedItemsType[] $selectedItems
     * @return \NOUT\Soap\StructType\CallingInfoType
     */
    public function setSelectedItems(?array $selectedItems = null): self
    {
        // validation for constraint: array
        if ('' !== ($selectedItemsArrayErrorMessage = self::validateSelectedItemsForArrayConstraintFromSetSelectedItems($selectedItems))) {
            throw new InvalidArgumentException($selectedItemsArrayErrorMessage, __LINE__);
        }
        $this->SelectedItems = $selectedItems;
        
        return $this;
    }
    /**
     * Add item to SelectedItems value
     * @throws InvalidArgumentException
     * @param \NOUT\Soap\StructType\SelectedItemsType $item
     * @return \NOUT\Soap\StructType\CallingInfoType
     */
    public function addToSelectedItems(\NOUT\Soap\StructType\SelectedItemsType $item): self
    {
        // validation for constraint: itemType
        if (!$item instanceof \NOUT\Soap\StructType\SelectedItemsType) {
            throw new InvalidArgumentException(sprintf('The SelectedItems property can only contain items of type \NOUT\Soap\StructType\SelectedItemsType, %s given', is_object($item) ? get_class($item) : (is_array($item) ? implode(', ', $item) : gettype($item))), __LINE__);
        }
        $this->SelectedItems[] = $item;
        
        return $this;
    }
}
