<?php

declare(strict_types=1);

namespace NOUT\Soap\StructType;

use InvalidArgumentException;
use NOUT\Soap\PackageBase\AbstractStructBase;

/**
 * This class stands for ColListType StructType
 * @subpackage Structs
 */
#[\AllowDynamicProperties]
class ColListType extends AbstractStructBase
{
    /**
     * The Col
     * Meta information extracted from the WSDL
     * - maxOccurs: unbounded
     * @var string[]
     */
    protected ?array $Col = null;
    /**
     * Constructor method for ColListType
     * @uses ColListType::setCol()
     * @param string[] $col
     */
    public function __construct(?array $col = null)
    {
        $this
            ->setCol($col);
    }
    /**
     * Get Col value
     * @return string[]
     */
    public function getCol(): ?array
    {
        return $this->Col;
    }
    /**
     * This method is responsible for validating the value(s) passed to the setCol method
     * This method is willingly generated in order to preserve the one-line inline validation within the setCol method
     * This has to validate that each item contained by the array match the itemType constraint
     * @param array $values
     * @return string A non-empty message if the values does not match the validation rules
     */
    public static function validateColForArrayConstraintFromSetCol(?array $values = []): string
    {
        if (!is_array($values)) {
            return '';
        }
        $message = '';
        $invalidValues = [];
        foreach ($values as $colListTypeColItem) {
            // validation for constraint: itemType
            if (!is_string($colListTypeColItem)) {
                $invalidValues[] = is_object($colListTypeColItem) ? get_class($colListTypeColItem) : sprintf('%s(%s)', gettype($colListTypeColItem), var_export($colListTypeColItem, true));
            }
        }
        if (!empty($invalidValues)) {
            $message = sprintf('The Col property can only contain items of type string, %s given', is_object($invalidValues) ? get_class($invalidValues) : (is_array($invalidValues) ? implode(', ', $invalidValues) : gettype($invalidValues)));
        }
        unset($invalidValues);
        
        return $message;
    }
    /**
     * Set Col value
     * @throws InvalidArgumentException
     * @param string[] $col
     * @return \NOUT\Soap\StructType\ColListType
     */
    public function setCol(?array $col = null): self
    {
        // validation for constraint: array
        if ('' !== ($colArrayErrorMessage = self::validateColForArrayConstraintFromSetCol($col))) {
            throw new InvalidArgumentException($colArrayErrorMessage, __LINE__);
        }
        $this->Col = $col;
        
        return $this;
    }
    /**
     * Add item to Col value
     * @throws InvalidArgumentException
     * @param string $item
     * @return \NOUT\Soap\StructType\ColListType
     */
    public function addToCol(string $item): self
    {
        // validation for constraint: itemType
        if (!is_string($item)) {
            throw new InvalidArgumentException(sprintf('The Col property can only contain items of type string, %s given', is_object($item) ? get_class($item) : (is_array($item) ? implode(', ', $item) : gettype($item))), __LINE__);
        }
        $this->Col[] = $item;
        
        return $this;
    }
}
