<?php

declare(strict_types=1);

namespace NOUT\Soap\StructType;

use InvalidArgumentException;
use NOUT\Soap\PackageBase\AbstractStructBase;

/**
 * This class stands for ColumnListType StructType
 * @subpackage Structs
 */
#[\AllowDynamicProperties]
class ColumnListType extends AbstractStructBase
{
    /**
     * The Column
     * Meta information extracted from the WSDL
     * - maxOccurs: unbounded
     * @var string[]
     */
    protected ?array $Column = null;
    /**
     * Constructor method for ColumnListType
     * @uses ColumnListType::setColumn()
     * @param string[] $column
     */
    public function __construct(?array $column = [])
    {
        $this
            ->setColumn($column);
    }
    /**
     * Get Column value
     * @return string[]
     */
    public function getColumn(): ?array
    {
        return $this->Column;
    }
    /**
     * This method is responsible for validating the value(s) passed to the setColumn method
     * This method is willingly generated in order to preserve the one-line inline validation within the setColumn method
     * This has to validate that each item contained by the array match the itemType constraint
     * @param array $values
     * @return string A non-empty message if the values does not match the validation rules
     */
    public static function validateColumnForArrayConstraintFromSetColumn(?array $values = []): string
    {
        if (!is_array($values)) {
            return '';
        }
        $message = '';
        $invalidValues = [];
        foreach ($values as $columnListTypeColumnItem) {
            // validation for constraint: itemType
            if (!is_string($columnListTypeColumnItem)) {
                $invalidValues[] = is_object($columnListTypeColumnItem) ? get_class($columnListTypeColumnItem) : sprintf('%s(%s)', gettype($columnListTypeColumnItem), var_export($columnListTypeColumnItem, true));
            }
        }
        if (!empty($invalidValues)) {
            $message = sprintf('The Column property can only contain items of type string, %s given', is_object($invalidValues) ? get_class($invalidValues) : (is_array($invalidValues) ? implode(', ', $invalidValues) : gettype($invalidValues)));
        }
        unset($invalidValues);
        
        return $message;
    }
    /**
     * Set Column value
     * @throws InvalidArgumentException
     * @param string[] $column
     * @return \NOUT\Soap\StructType\ColumnListType
     */
    public function setColumn(?array $column = null): self
    {
        // validation for constraint: array
        if ('' !== ($columnArrayErrorMessage = self::validateColumnForArrayConstraintFromSetColumn($column))) {
            throw new InvalidArgumentException($columnArrayErrorMessage, __LINE__);
        }
        $this->Column = $column;
        
        return $this;
    }
    /**
     * Add item to Column value
     * @throws InvalidArgumentException
     * @param string $item
     * @return \NOUT\Soap\StructType\ColumnListType
     */
    public function addToColumn(string $item): self
    {
        // validation for constraint: itemType
        if (!is_string($item)) {
            throw new InvalidArgumentException(sprintf('The Column property can only contain items of type string, %s given', is_object($item) ? get_class($item) : (is_array($item) ? implode(', ', $item) : gettype($item))), __LINE__);
        }
        $this->Column[] = $item;
        
        return $this;
    }
}
