<?php

declare(strict_types=1);

namespace NOUT\Soap\StructType;

use InvalidArgumentException;
use NOUT\Entity\UsernameToken\UsernameTokenInterface;
use NOUT\Soap\PackageBase\AbstractStructBase;

/**
 * This class stands for GetTokenSession StructType
 * @subpackage Structs
 */
#[\AllowDynamicProperties]
class GetTokenSession extends AbstractStructBase
{
    /**
     * The Token
     * Meta information extracted from the WSDL
     * - choice: Token | UsernameToken
     * - choiceMaxOccurs: 1
     * - choiceMinOccurs: 1
     * @var string|null
     */
    protected ?string $Token = null;
    /**
     * The UsernameToken
     * Meta information extracted from the WSDL
     * - choice: Token | UsernameToken
     * - choiceMaxOccurs: 1
     * - choiceMinOccurs: 1
     * @var UsernameTokenInterface|null
     */
    protected ?UsernameTokenInterface $UsernameToken = null;
    /**
     * The ExtranetUser
     * Meta information extracted from the WSDL
     * - minOccurs: 0
     * @var \NOUT\Soap\StructType\ExtranetUserType|null
     */
    protected ?\NOUT\Soap\StructType\ExtranetUserType $ExtranetUser = null;
    /**
     * The DefaultClientLanguageCode
     * @var int|null
     */
    protected ?int $DefaultClientLanguageCode = null;
    /**
     * The ForConfiguration
     * Meta information extracted from the WSDL
     * - minOccurs: 0
     * @var int|null
     */
    protected ?int $ForConfiguration = null;
    /**
     * Constructor method for GetTokenSession
     * @uses GetTokenSession::setToken()
     * @uses GetTokenSession::setUsernameToken()
     * @uses GetTokenSession::setExtranetUser()
     * @uses GetTokenSession::setDefaultClientLanguageCode()
     * @uses GetTokenSession::setForConfiguration()
     * @param string|null $token
     * @param UsernameTokenInterface|null $usernameToken
     * @param ExtranetUserType|null $extranetUser
     * @param int|null $defaultClientLanguageCode
     * @param int|null $forConfiguration
     */
    public function __construct(?string $token = null, ?UsernameTokenInterface $usernameToken = null, ?ExtranetUserType $extranetUser = null, ?int $defaultClientLanguageCode = null, ?int $forConfiguration = null)
    {
        $this
            ->setToken($token)
            ->setUsernameToken($usernameToken)
            ->setExtranetUser($extranetUser)
            ->setDefaultClientLanguageCode($defaultClientLanguageCode)
            ->setForConfiguration($forConfiguration);
    }
    /**
     * Get Token value
     * @return string|null
     */
    public function getToken(): ?string
    {
        return $this->Token ?? null;
    }
    /**
     * This method is responsible for validating the value(s) passed to the setToken method
     * This method is willingly generated in order to preserve the one-line inline validation within the setToken method
     * This has to validate that the property which is being set is the only one among the given choices
     * @param mixed $value
     * @return string A non-empty message if the values does not match the validation rules
     */
    public function validateTokenForChoiceConstraintFromSetToken($value): string
    {
        $message = '';
        if (is_null($value)) {
            return $message;
        }
        $properties = [
            'UsernameToken',
        ];
        try {
            foreach ($properties as $property) {
                if (isset($this->{$property})) {
                    throw new InvalidArgumentException(sprintf('The property Token can\'t be set as the property %s is already set. Only one property must be set among these properties: Token, %s.', $property, implode(', ', $properties)), __LINE__);
                }
            }
        } catch (InvalidArgumentException $e) {
            $message = $e->getMessage();
        }
        
        return $message;
    }
    /**
     * Set Token value
     * This property belongs to a choice that allows only one property to exist. It is
     * therefore removable from the request, consequently if the value assigned to this
     * property is null, the property is removed from this object
     * @throws InvalidArgumentException
     * @param string $token
     * @return \NOUT\Soap\StructType\GetTokenSession
     */
    public function setToken(?string $token = null): self
    {
        // validation for constraint: string
        if (!is_null($token) && !is_string($token)) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide a string, %s given', var_export($token, true), gettype($token)), __LINE__);
        }
        // validation for constraint: choice(Token, UsernameToken)
        if ('' !== ($tokenChoiceErrorMessage = self::validateTokenForChoiceConstraintFromSetToken($token))) {
            throw new InvalidArgumentException($tokenChoiceErrorMessage, __LINE__);
        }
        if (is_null($token) || (is_array($token) && empty($token))) {
            unset($this->Token);
        } else {
            $this->Token = $token;
        }
        
        return $this;
    }
    /**
     * Get UsernameToken value
     * @return \NOUT\Soap\StructType\UsernameTokenType|null
     */
    public function getUsernameToken(): ?\NOUT\Soap\StructType\UsernameTokenType
    {
        return $this->UsernameToken ?? null;
    }
    /**
     * This method is responsible for validating the value(s) passed to the setUsernameToken method
     * This method is willingly generated in order to preserve the one-line inline validation within the setUsernameToken method
     * This has to validate that the property which is being set is the only one among the given choices
     * @param mixed $value
     * @return string A non-empty message if the values does not match the validation rules
     */
    public function validateUsernameTokenForChoiceConstraintFromSetUsernameToken($value): string
    {
        $message = '';
        if (is_null($value)) {
            return $message;
        }
        $properties = [
            'Token',
        ];
        try {
            foreach ($properties as $property) {
                if (isset($this->{$property})) {
                    throw new InvalidArgumentException(sprintf('The property UsernameToken can\'t be set as the property %s is already set. Only one property must be set among these properties: UsernameToken, %s.', $property, implode(', ', $properties)), __LINE__);
                }
            }
        } catch (InvalidArgumentException $e) {
            $message = $e->getMessage();
        }
        
        return $message;
    }
    /**
     * Set UsernameToken value
     * This property belongs to a choice that allows only one property to exist. It is
     * therefore removable from the request, consequently if the value assigned to this
     * property is null, the property is removed from this object
     * @throws InvalidArgumentException
     * @param UsernameTokenInterface|null $usernameToken
     * @return \NOUT\Soap\StructType\GetTokenSession
     */
    public function setUsernameToken(?UsernameTokenInterface $usernameToken = null): self
    {
        // validation for constraint: choice(Token, UsernameToken)
        if ('' !== ($usernameTokenChoiceErrorMessage = self::validateUsernameTokenForChoiceConstraintFromSetUsernameToken($usernameToken))) {
            throw new InvalidArgumentException($usernameTokenChoiceErrorMessage, __LINE__);
        }
        if (is_null($usernameToken) || (is_array($usernameToken) && empty($usernameToken))) {
            unset($this->UsernameToken);
        } else {
            $this->UsernameToken = $usernameToken;
        }
        
        return $this;
    }
    /**
     * Get ExtranetUser value
     * @return \NOUT\Soap\StructType\ExtranetUserType|null
     */
    public function getExtranetUser(): ?\NOUT\Soap\StructType\ExtranetUserType
    {
        return $this->ExtranetUser;
    }
    /**
     * Set ExtranetUser value
     * @param \NOUT\Soap\StructType\ExtranetUserType $extranetUser
     * @return \NOUT\Soap\StructType\GetTokenSession
     */
    public function setExtranetUser(?\NOUT\Soap\StructType\ExtranetUserType $extranetUser = null): self
    {
        $this->ExtranetUser = $extranetUser;
        
        return $this;
    }
    /**
     * Get DefaultClientLanguageCode value
     * @return int|null
     */
    public function getDefaultClientLanguageCode(): ?int
    {
        return $this->DefaultClientLanguageCode;
    }
    /**
     * Set DefaultClientLanguageCode value
     * @param int $defaultClientLanguageCode
     * @return \NOUT\Soap\StructType\GetTokenSession
     */
    public function setDefaultClientLanguageCode(?int $defaultClientLanguageCode = null): self
    {
        // validation for constraint: int
        if (!is_null($defaultClientLanguageCode) && !(is_int($defaultClientLanguageCode) || ctype_digit($defaultClientLanguageCode))) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide an integer value, %s given', var_export($defaultClientLanguageCode, true), gettype($defaultClientLanguageCode)), __LINE__);
        }
        $this->DefaultClientLanguageCode = $defaultClientLanguageCode;
        
        return $this;
    }
    /**
     * Get ForConfiguration value
     * @return int|null
     */
    public function getForConfiguration(): ?int
    {
        return $this->ForConfiguration;
    }
    /**
     * Set ForConfiguration value
     * @param int $forConfiguration
     * @return \NOUT\Soap\StructType\GetTokenSession
     */
    public function setForConfiguration(?int $forConfiguration = null): self
    {
        // validation for constraint: int
        if (!is_null($forConfiguration) && !(is_int($forConfiguration) || ctype_digit($forConfiguration))) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide an integer value, %s given', var_export($forConfiguration, true), gettype($forConfiguration)), __LINE__);
        }
        $this->ForConfiguration = $forConfiguration;
        
        return $this;
    }
}
