<?php

declare(strict_types=1);

namespace NOUT\Soap\StructType;

use InvalidArgumentException;
use NOUT\Soap\PackageBase\AbstractStructBase;

/**
 * This class stands for RecipientCheck StructType
 * @subpackage Structs
 */
#[\AllowDynamicProperties]
class RecipientCheck extends AbstractStructBase
{
    /**
     * The To
     * Meta information extracted from the WSDL
     * - maxOccurs: unbounded
     * - minOccurs: 0
     * @var string[]
     */
    protected ?array $To = null;
    /**
     * The Cc
     * Meta information extracted from the WSDL
     * - maxOccurs: unbounded
     * - minOccurs: 0
     * @var string[]
     */
    protected ?array $Cc = null;
    /**
     * The Cci
     * Meta information extracted from the WSDL
     * - maxOccurs: unbounded
     * - minOccurs: 0
     * @var string[]
     */
    protected ?array $Cci = null;
    /**
     * Constructor method for RecipientCheck
     * @uses RecipientCheck::setTo()
     * @uses RecipientCheck::setCc()
     * @uses RecipientCheck::setCci()
     * @param string[] $to
     * @param string[] $cc
     * @param string[] $cci
     */
    public function __construct(?array $to = null, ?array $cc = null, ?array $cci = null)
    {
        $this
            ->setTo($to)
            ->setCc($cc)
            ->setCci($cci);
    }
    /**
     * Get To value
     * @return string[]
     */
    public function getTo(): ?array
    {
        return $this->To;
    }
    /**
     * This method is responsible for validating the value(s) passed to the setTo method
     * This method is willingly generated in order to preserve the one-line inline validation within the setTo method
     * This has to validate that each item contained by the array match the itemType constraint
     * @param array $values
     * @return string A non-empty message if the values does not match the validation rules
     */
    public static function validateToForArrayConstraintFromSetTo(?array $values = []): string
    {
        if (!is_array($values)) {
            return '';
        }
        $message = '';
        $invalidValues = [];
        foreach ($values as $recipientCheckToItem) {
            // validation for constraint: itemType
            if (!is_string($recipientCheckToItem)) {
                $invalidValues[] = is_object($recipientCheckToItem) ? get_class($recipientCheckToItem) : sprintf('%s(%s)', gettype($recipientCheckToItem), var_export($recipientCheckToItem, true));
            }
        }
        if (!empty($invalidValues)) {
            $message = sprintf('The To property can only contain items of type string, %s given', is_object($invalidValues) ? get_class($invalidValues) : (is_array($invalidValues) ? implode(', ', $invalidValues) : gettype($invalidValues)));
        }
        unset($invalidValues);
        
        return $message;
    }
    /**
     * Set To value
     * @throws InvalidArgumentException
     * @param string[] $to
     * @return \NOUT\Soap\StructType\RecipientCheck
     */
    public function setTo(?array $to = null): self
    {
        // validation for constraint: array
        if ('' !== ($toArrayErrorMessage = self::validateToForArrayConstraintFromSetTo($to))) {
            throw new InvalidArgumentException($toArrayErrorMessage, __LINE__);
        }
        $this->To = $to;
        
        return $this;
    }
    /**
     * Add item to To value
     * @throws InvalidArgumentException
     * @param string $item
     * @return \NOUT\Soap\StructType\RecipientCheck
     */
    public function addToTo(string $item): self
    {
        // validation for constraint: itemType
        if (!is_string($item)) {
            throw new InvalidArgumentException(sprintf('The To property can only contain items of type string, %s given', is_object($item) ? get_class($item) : (is_array($item) ? implode(', ', $item) : gettype($item))), __LINE__);
        }
        $this->To[] = $item;
        
        return $this;
    }
    /**
     * Get Cc value
     * @return string[]
     */
    public function getCc(): ?array
    {
        return $this->Cc;
    }
    /**
     * This method is responsible for validating the value(s) passed to the setCc method
     * This method is willingly generated in order to preserve the one-line inline validation within the setCc method
     * This has to validate that each item contained by the array match the itemType constraint
     * @param array $values
     * @return string A non-empty message if the values does not match the validation rules
     */
    public static function validateCcForArrayConstraintFromSetCc(?array $values = []): string
    {
        if (!is_array($values)) {
            return '';
        }
        $message = '';
        $invalidValues = [];
        foreach ($values as $recipientCheckCcItem) {
            // validation for constraint: itemType
            if (!is_string($recipientCheckCcItem)) {
                $invalidValues[] = is_object($recipientCheckCcItem) ? get_class($recipientCheckCcItem) : sprintf('%s(%s)', gettype($recipientCheckCcItem), var_export($recipientCheckCcItem, true));
            }
        }
        if (!empty($invalidValues)) {
            $message = sprintf('The Cc property can only contain items of type string, %s given', is_object($invalidValues) ? get_class($invalidValues) : (is_array($invalidValues) ? implode(', ', $invalidValues) : gettype($invalidValues)));
        }
        unset($invalidValues);
        
        return $message;
    }
    /**
     * Set Cc value
     * @throws InvalidArgumentException
     * @param string[] $cc
     * @return \NOUT\Soap\StructType\RecipientCheck
     */
    public function setCc(?array $cc = null): self
    {
        // validation for constraint: array
        if ('' !== ($ccArrayErrorMessage = self::validateCcForArrayConstraintFromSetCc($cc))) {
            throw new InvalidArgumentException($ccArrayErrorMessage, __LINE__);
        }
        $this->Cc = $cc;
        
        return $this;
    }
    /**
     * Add item to Cc value
     * @throws InvalidArgumentException
     * @param string $item
     * @return \NOUT\Soap\StructType\RecipientCheck
     */
    public function addToCc(string $item): self
    {
        // validation for constraint: itemType
        if (!is_string($item)) {
            throw new InvalidArgumentException(sprintf('The Cc property can only contain items of type string, %s given', is_object($item) ? get_class($item) : (is_array($item) ? implode(', ', $item) : gettype($item))), __LINE__);
        }
        $this->Cc[] = $item;
        
        return $this;
    }
    /**
     * Get Cci value
     * @return string[]
     */
    public function getCci(): ?array
    {
        return $this->Cci;
    }
    /**
     * This method is responsible for validating the value(s) passed to the setCci method
     * This method is willingly generated in order to preserve the one-line inline validation within the setCci method
     * This has to validate that each item contained by the array match the itemType constraint
     * @param array $values
     * @return string A non-empty message if the values does not match the validation rules
     */
    public static function validateCciForArrayConstraintFromSetCci(?array $values = []): string
    {
        if (!is_array($values)) {
            return '';
        }
        $message = '';
        $invalidValues = [];
        foreach ($values as $recipientCheckCciItem) {
            // validation for constraint: itemType
            if (!is_string($recipientCheckCciItem)) {
                $invalidValues[] = is_object($recipientCheckCciItem) ? get_class($recipientCheckCciItem) : sprintf('%s(%s)', gettype($recipientCheckCciItem), var_export($recipientCheckCciItem, true));
            }
        }
        if (!empty($invalidValues)) {
            $message = sprintf('The Cci property can only contain items of type string, %s given', is_object($invalidValues) ? get_class($invalidValues) : (is_array($invalidValues) ? implode(', ', $invalidValues) : gettype($invalidValues)));
        }
        unset($invalidValues);
        
        return $message;
    }
    /**
     * Set Cci value
     * @throws InvalidArgumentException
     * @param string[] $cci
     * @return \NOUT\Soap\StructType\RecipientCheck
     */
    public function setCci(?array $cci = null): self
    {
        // validation for constraint: array
        if ('' !== ($cciArrayErrorMessage = self::validateCciForArrayConstraintFromSetCci($cci))) {
            throw new InvalidArgumentException($cciArrayErrorMessage, __LINE__);
        }
        $this->Cci = $cci;
        
        return $this;
    }
    /**
     * Add item to Cci value
     * @throws InvalidArgumentException
     * @param string $item
     * @return \NOUT\Soap\StructType\RecipientCheck
     */
    public function addToCci(string $item): self
    {
        // validation for constraint: itemType
        if (!is_string($item)) {
            throw new InvalidArgumentException(sprintf('The Cci property can only contain items of type string, %s given', is_object($item) ? get_class($item) : (is_array($item) ? implode(', ', $item) : gettype($item))), __LINE__);
        }
        $this->Cci[] = $item;
        
        return $this;
    }
}
