<?php

declare(strict_types=1);

namespace NOUT\Soap\StructType;

use InvalidArgumentException;
use NOUT\Soap\PackageBase\AbstractStructBase;

/**
 * This class stands for Request StructType
 * @subpackage Structs
 */
#[\AllowDynamicProperties]
class Request extends AbstractStructBase
{
    /**
     * The Table
     * @var string|null
     */
    protected ?string $Table = null;
    /**
     * The ColList
     * Meta information extracted from the WSDL
     * - minOccurs: 0
     * @var \NOUT\Soap\StructType\ColListType|null
     */
    protected ?\NOUT\Soap\StructType\ColListType $ColList = null;
    /**
     * The CondList
     * @var string|null
     */
    protected ?string $CondList = null;
    /**
     * The MaxResult
     * Meta information extracted from the WSDL
     * - minOccurs: 0
     * @var int|null
     */
    protected ?int $MaxResult = null;
    /**
     * The Sort1
     * Meta information extracted from the WSDL
     * - minOccurs: 0
     * @var \NOUT\Soap\StructType\SortType|null
     */
    protected ?\NOUT\Soap\StructType\SortType $Sort1 = null;
    /**
     * The Sort2
     * Meta information extracted from the WSDL
     * - minOccurs: 0
     * @var \NOUT\Soap\StructType\SortType|null
     */
    protected ?\NOUT\Soap\StructType\SortType $Sort2 = null;
    /**
     * The Sort3
     * Meta information extracted from the WSDL
     * - minOccurs: 0
     * @var \NOUT\Soap\StructType\SortType|null
     */
    protected ?\NOUT\Soap\StructType\SortType $Sort3 = null;
    /**
     * The CallingColumn
     * Meta information extracted from the WSDL
     * - minOccurs: 0
     * @var string|null
     */
    protected ?string $CallingColumn = null;
    /**
     * The CallingInfo
     * Meta information extracted from the WSDL
     * - minOccurs: 0
     * @var \NOUT\Soap\StructType\CallingInfoType|null
     */
    protected ?\NOUT\Soap\StructType\CallingInfoType $CallingInfo = null;
    /**
     * Constructor method for Request
     * @uses Request::setTable()
     * @uses Request::setColList()
     * @uses Request::setCondList()
     * @uses Request::setMaxResult()
     * @uses Request::setSort1()
     * @uses Request::setSort2()
     * @uses Request::setSort3()
     * @uses Request::setCallingColumn()
     * @uses Request::setCallingInfo()
     * @param string $table
     * @param \NOUT\Soap\StructType\ColListType $colList
     * @param string $condList
     * @param int $maxResult
     * @param \NOUT\Soap\StructType\SortType $sort1
     * @param \NOUT\Soap\StructType\SortType $sort2
     * @param \NOUT\Soap\StructType\SortType $sort3
     * @param string $callingColumn
     * @param \NOUT\Soap\StructType\CallingInfoType $callingInfo
     */
    public function __construct(?string $table = null, ?\NOUT\Soap\StructType\ColListType $colList = null, ?string $condList = null, ?int $maxResult = null, ?\NOUT\Soap\StructType\SortType $sort1 = null, ?\NOUT\Soap\StructType\SortType $sort2 = null, ?\NOUT\Soap\StructType\SortType $sort3 = null, ?string $callingColumn = null, ?\NOUT\Soap\StructType\CallingInfoType $callingInfo = null)
    {
        $this
            ->setTable($table)
            ->setColList($colList)
            ->setCondList($condList)
            ->setMaxResult($maxResult)
            ->setSort1($sort1)
            ->setSort2($sort2)
            ->setSort3($sort3)
            ->setCallingColumn($callingColumn)
            ->setCallingInfo($callingInfo);
    }
    /**
     * Get Table value
     * @return string|null
     */
    public function getTable(): ?string
    {
        return $this->Table;
    }
    /**
     * Set Table value
     * @param string $table
     * @return \NOUT\Soap\StructType\Request
     */
    public function setTable(?string $table = null): self
    {
        // validation for constraint: string
        if (!is_null($table) && !is_string($table)) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide a string, %s given', var_export($table, true), gettype($table)), __LINE__);
        }
        $this->Table = $table;
        
        return $this;
    }
    /**
     * Get ColList value
     * @return \NOUT\Soap\StructType\ColListType|null
     */
    public function getColList(): ?\NOUT\Soap\StructType\ColListType
    {
        return $this->ColList;
    }
    /**
     * Set ColList value
     * @param \NOUT\Soap\StructType\ColListType $colList
     * @return \NOUT\Soap\StructType\Request
     */
    public function setColList(?\NOUT\Soap\StructType\ColListType $colList = null): self
    {
        $this->ColList = $colList;
        
        return $this;
    }
    /**
     * Get CondList value
     * @return string|null
     */
    public function getCondList(): ?string
    {
        return $this->CondList;
    }
    /**
     * Set CondList value
     * @param string $condList
     * @return \NOUT\Soap\StructType\Request
     */
    public function setCondList(?string $condList = null): self
    {
        // validation for constraint: string
        if (!is_null($condList) && !is_string($condList)) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide a string, %s given', var_export($condList, true), gettype($condList)), __LINE__);
        }
        $this->CondList = $condList;
        
        return $this;
    }
    /**
     * Get MaxResult value
     * @return int|null
     */
    public function getMaxResult(): ?int
    {
        return $this->MaxResult;
    }
    /**
     * Set MaxResult value
     * @param int $maxResult
     * @return \NOUT\Soap\StructType\Request
     */
    public function setMaxResult(?int $maxResult = null): self
    {
        // validation for constraint: int
        if (!is_null($maxResult) && !(is_int($maxResult) || ctype_digit($maxResult))) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide an integer value, %s given', var_export($maxResult, true), gettype($maxResult)), __LINE__);
        }
        $this->MaxResult = $maxResult;
        
        return $this;
    }
    /**
     * Get Sort1 value
     * @return \NOUT\Soap\StructType\SortType|null
     */
    public function getSort1(): ?\NOUT\Soap\StructType\SortType
    {
        return $this->Sort1;
    }
    /**
     * Set Sort1 value
     * @param \NOUT\Soap\StructType\SortType $sort1
     * @return \NOUT\Soap\StructType\Request
     */
    public function setSort1(?\NOUT\Soap\StructType\SortType $sort1 = null): self
    {
        $this->Sort1 = $sort1;
        
        return $this;
    }
    /**
     * Get Sort2 value
     * @return \NOUT\Soap\StructType\SortType|null
     */
    public function getSort2(): ?\NOUT\Soap\StructType\SortType
    {
        return $this->Sort2;
    }
    /**
     * Set Sort2 value
     * @param \NOUT\Soap\StructType\SortType $sort2
     * @return \NOUT\Soap\StructType\Request
     */
    public function setSort2(?\NOUT\Soap\StructType\SortType $sort2 = null): self
    {
        $this->Sort2 = $sort2;
        
        return $this;
    }
    /**
     * Get Sort3 value
     * @return \NOUT\Soap\StructType\SortType|null
     */
    public function getSort3(): ?\NOUT\Soap\StructType\SortType
    {
        return $this->Sort3;
    }
    /**
     * Set Sort3 value
     * @param \NOUT\Soap\StructType\SortType $sort3
     * @return \NOUT\Soap\StructType\Request
     */
    public function setSort3(?\NOUT\Soap\StructType\SortType $sort3 = null): self
    {
        $this->Sort3 = $sort3;
        
        return $this;
    }
    /**
     * Get CallingColumn value
     * @return string|null
     */
    public function getCallingColumn(): ?string
    {
        return $this->CallingColumn;
    }
    /**
     * Set CallingColumn value
     * @param string $callingColumn
     * @return \NOUT\Soap\StructType\Request
     */
    public function setCallingColumn(?string $callingColumn = null): self
    {
        // validation for constraint: string
        if (!is_null($callingColumn) && !is_string($callingColumn)) {
            throw new InvalidArgumentException(sprintf('Invalid value %s, please provide a string, %s given', var_export($callingColumn, true), gettype($callingColumn)), __LINE__);
        }
        $this->CallingColumn = $callingColumn;
        
        return $this;
    }
    /**
     * Get CallingInfo value
     * @return \NOUT\Soap\StructType\CallingInfoType|null
     */
    public function getCallingInfo(): ?\NOUT\Soap\StructType\CallingInfoType
    {
        return $this->CallingInfo;
    }
    /**
     * Set CallingInfo value
     * @param \NOUT\Soap\StructType\CallingInfoType $callingInfo
     * @return \NOUT\Soap\StructType\Request
     */
    public function setCallingInfo(?\NOUT\Soap\StructType\CallingInfoType $callingInfo = null): self
    {
        $this->CallingInfo = $callingInfo;
        
        return $this;
    }
}
