<?php
if (PHP_VERSION_ID < 70400){
    http_response_code (500);
    header('Content-Type: application/json');
    $rep = new \stdClass();
    $rep->status = 'error';
    $rep->error = '9999';
    $rep->msg = 'need php version >= 7.4';
    echo json_encode($rep);
    die ;
}

$isSoapExtension = extension_loaded('soap');
if (!$isSoapExtension) {
    http_response_code (500);
    header('Content-Type: application/json');
    $rep = new \stdClass();
    $rep->status = 'error';
    $rep->error = '9998';
    $rep->msg = 'soap extension is not enabled';
    echo json_encode($rep);
    die ;
}

const DEBUG_MODE = false; //mode debug.
$mode_debug = DEBUG_MODE || file_exists('active_debug.txt');

if (!$mode_debug)
{
    error_reporting(0);
    ini_set("display_errors", 0);
}
else
{
    ini_set('display_errors', 1);
    ini_set('display_startup_errors', 1);
    error_reporting(E_ERROR | E_PARSE);
}
ini_set('max_execution_time', 0);

//autoload
require_once __DIR__ . '/vendor/autoload.php';

//par defaut, pas d'info config ou noutonline,  c'est inclus par config.php et noutonline.php respectivement, noutonline etant optionnel
$config = [];
$noutonline = [];

require_once './_function.php';
require_once './config.php';
if (file_exists('./noutonline.php')){
    require_once './noutonline.php';
}

$debug = new NOUT\DebugTrace($mode_debug);
$debug->startTrace();



//vérification de l'apikey, c'est lié au point d'entree et pas au NOUTOnline
/** @var array $config chargé par require ./config.php */
if (!executeSecurity($config, $debug)) {
    die; //les erreurs sont gérées dans la fonction
}

$rep = new \stdClass();
//récupération de l'id de l'action et des infos de connexion
$idaction = getIDAction($config);
if (empty($idaction))
{
    $debug->writeTrace('action non trouve');
    $debug->endTrace();
    if (array_key_exists('404', $config) && !empty($config['404'])){
        if (is_callable($config['404'])){
            $config['404']();
            die();
        }

        if (file_exists($config['404'])){
            http_response_code (404);
            header('Content-Type: '.mime_content_type($config['404']));
            echo file_get_contents($config['404']);
            die;
        }
    }

    http_response_code (404);
    $rep->status = 'error';
    $rep->error = '5';
    $rep->msg = (!array_key_exists('endpoints', $config) || empty($config['endpoints'])) ? 'not configured' : 'not found';

    header('Content-Type: application/json');
    echo json_encode($rep);
    die ;
}

$infoCnx = getInfoCnx($config, $noutonline);
//vérification de la configuration
if (    empty($infoCnx['username'])
    || !isset($infoCnx['password'])
){
    $debug->writeTrace('bad configuration');
    $debug->endTrace();

    http_response_code (403);
    $rep->status = 'error';
    $rep->error = '4';
    $rep->msg = 'not allowed';


    if ($mode_debug){
        $rep->debug = new \stdClass();
        $rep->debug->idaction = $idaction;
        $rep->debug->username = $infoCnx['username'] ?? null;
        $rep->debug->password = $infoCnx['password'] ?? null;
        $rep->debug->check_username = empty($infoCnx['username']);
        $rep->debug->check_password = !isset($infoCnx['password']);
        $rep->debug->check_idaction = empty($idaction);
    }
    header('Content-Type: application/json');
    echo json_encode($rep);
    die ;
}
//récupération du type d'action
$client_class = getClientClass($config);
$client = new $client_class($infoCnx);
$client->setDebugTrace($debug);
if ($config['with_log'] ?? false){
    $client->setLogSIMAX($config['log_action'] ?? null, $config['log_parameter'] ?? null);
}
//récupération des paramètres
$params = getParams($config, $client, $mode_debug, $debug);

//connexion
try {
    $client->bConnecte();
}
catch (\Exception $e)
{
    $debug->endTrace();
    onCnxException($config, $rep, $e, 'on_cnx_exception');

    die;
}
//execution de l'action
try{
    $retExec = $client->execute($idaction, $params);
    $rep->status = 'ok';

    if (!empty($retExec)){ //transformation du retour
        $rep->result = transformeRetour($config, $retExec);
    }
}
catch (\Exception $e)
{
    $client->bDisconnect();
    $debug->endTrace();
    onCnxException($config, $rep, $e, 'on_action_exception');
    die;
}

//déconnexion
try {
    $client->bDisconnect();
}
catch (\Exception $e)
{
    //on fait rien sur la deconnection
}

$debug->endTrace();
//envoi du retour
envoiRetour($config, $rep);


